import {
  View,
  Text,
  FlatList,
  Pressable,
  ScrollView,
  Alert,
  Platform,
} from 'react-native';
import React, { FC, useEffect, useState } from 'react';
import { resetLogItems, toggleDebugger, useDebugItems } from './utils';
import Clipboard from '@react-native-clipboard/clipboard';
import { useShowDebugger } from './utils/useShowDebugger';
import { DebuggerItem } from './components/DebuggerItem';
import { styles, itemColorByType } from './styles';
import { DebuggerState } from './state';
import { pickDirectory } from 'react-native-document-picker';
import { writeFile } from 'react-native-saf-x';
import RnBlobUtil from 'react-native-blob-util';

export * from './utils';
export * from './components';

export function multiply(a: number, b: number): Promise<number> {
  return Promise.resolve(a * b);
}

const Item = ({ item }: any) => {
  const [visible, setVisible] = useState(false);

  const handleCopyPress = () => Clipboard.setString(JSON.stringify(item));

  const colorByType = itemColorByType(item.logType);

  const textColor = { color: colorByType };
  const iconColor = { backgroundColor: colorByType };

  return (
    <View style={styles.itemWrapper}>
      <View style={styles.itemHeader}>
        <View style={[styles.itemIcon, iconColor]} />
        <View>
          <Text selectable style={[styles.itemLabel, textColor]}>
            {item.label}
          </Text>
          <Text selectable style={[styles.itemTime, textColor]}>
            {item.timestamp}
          </Text>
          {Boolean(item.type) && (
            <Text selectable style={[styles.itemType, textColor]}>
              {item.type}
            </Text>
          )}
        </View>
      </View>
      <ScrollView
        style={
          visible ? styles.itemScrollWrapperVisible : styles.itemScrollWrapper
        }
        contentContainerStyle={item.itemScroll}
        horizontal
      >
        <DebuggerItem onToggle={setVisible} item={item.data} />
      </ScrollView>
      {visible && (
        <Pressable onPress={handleCopyPress} style={styles.itemCopyBtn}>
          <Text style={styles.itemCopyText}>Copy</Text>
        </Pressable>
      )}
    </View>
  );
};

const Separator = () => <View style={styles.separator} />;

const EmptySection = () => (
  <View style={styles.emptyWrapper}>
    <Text style={styles.emptyText}>No events captured yet</Text>
  </View>
);

const keyExtractor = ({ id }: { id: string | number }) => `${id}`;

type Props = {
  getGlobalState?: () => Record<any, any>;
};

const renderItem = ({ item }: any) => <Item item={item} />;

export const Debugger: FC<Props> = React.memo(({ getGlobalState }) => {
  const [debugItems] = useDebugItems();
  const [show] = useShowDebugger();
  console.log({ debugItems });
  useEffect(() => {
    DebuggerState.isMounted = true;
    return () => {
      DebuggerState.isMounted = false;
    };
  }, []);

  useEffect(() => {
    DebuggerState.getGlobalState = getGlobalState;
  }, [getGlobalState]);

  const handleClose = () => toggleDebugger(false);

  const handleGetGlobalState = () =>
    getGlobalState && Clipboard.setString(JSON.stringify(getGlobalState?.()));

  const handleExportItems = async () => {
    try {
      /*fetch('https://pastebin.com/api/api_post.php', {
        body: `api_dev_key=clcVbOdfoqzhASuaHaFSLXaiG78XAeRz&api_paste_code=${JSON.stringify(
          debugItems
        )}&api_option=paste&api_paste_private=2&api_user_key=37eb3c9cdd035fbb30c3f1b5a9e4b69b`,
        headers: {
          'Content-Type': 'application/x-www-form-urlencoded',
        },
        method: 'POST',
      })
        .then((d) => d.text())
        .then((url) => {
          Clipboard.setString(url);
          Alert.alert(url);
        });*/
      pickDirectory()
        .then((dir) => {
          if (!dir) {
            Clipboard.setString(JSON.stringify(debugItems));
            Alert.alert('File saving failed, copied instead');
            return;
          }
          const filePath = `${
            dir.uri
          }/qa-debugger-${new Date().toISOString()}.txt`;
          console.log({ filePath });
          if (Platform.OS === 'android' && dir.uri?.startsWith('content://')) {
            writeFile(filePath, JSON.stringify(debugItems))
              .then(() => {
                Alert.alert('File successfully saved');
              })
              .catch(() => {
                Clipboard.setString(JSON.stringify(debugItems));
                Alert.alert('File saving failed, copied instead');
              });
            return;
          }
          RnBlobUtil.fs
            .writeFile(filePath, JSON.stringify(debugItems), 'utf8')
            .then(() => {
              Alert.alert('File successfully saved');
            })
            .catch(() => {
              Clipboard.setString(JSON.stringify(debugItems));
              Alert.alert('File saving failed, copied instead');
            });
        })
        .catch(() => {
          Clipboard.setString(JSON.stringify(debugItems));
          Alert.alert('Failed to pick a directory, copied instead');
        });
    } catch {
      Clipboard.setString(JSON.stringify(debugItems));
      Alert.alert('Items copied');
    }
  };

  return show ? (
    <View style={styles.wrapper}>
      <View style={styles.titleWrapper}>
        <Text style={styles.title}>QA DEBUGGER</Text>
      </View>
      <FlatList
        keyExtractor={keyExtractor}
        contentContainerStyle={styles.flex}
        ItemSeparatorComponent={Separator}
        data={debugItems}
        ListEmptyComponent={EmptySection}
        renderItem={renderItem}
      />
      <View style={styles.buttonSection}>
        <Pressable onPress={handleExportItems} style={styles.button}>
          <Text style={styles.buttonText}>Export items</Text>
        </Pressable>
        <Pressable
          onPress={resetLogItems}
          style={[styles.button, styles.centralButton]}
        >
          <Text style={styles.buttonText}>Clear</Text>
        </Pressable>
        <Pressable onPress={handleClose} style={styles.button}>
          <Text style={styles.buttonText}>Close</Text>
        </Pressable>
      </View>
      {Boolean(getGlobalState) && (
        <View style={styles.buttonSection}>
          <Pressable onPress={handleGetGlobalState} style={styles.button}>
            <Text style={styles.buttonText}>Copy global state</Text>
          </Pressable>
        </View>
      )}
    </View>
  ) : null;
});
